@@ -8,7 +8,7 @@
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 namespace itpp {
 
   /*!
-    \brief Checks if a filename already exists on the disk
+    \brief Checks if a file named \c name already exists on the disk
     \ingroup itfile
   */
   bool exist(const std::string& name);
@@ -60,7 +60,7 @@ namespace itpp {
       and "Big Endian" (\c b_endian ).
 
       "Little Endian" means that the low-order byte of the number is stored
-      at the lowest adress (i.e. the little end comes first). "Big Endian"
+      at the lowest address (i.e. the little end comes first). "Big Endian"
       means that the high-order byte of the number is stored in memory at
       the lowest address (i.e. the big end comes first)
     */
