@@ -8,7 +8,7 @@
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,30 @@
 
 //! \cond
 
+#ifndef HAVE_RINT
+double rint(double x)
+{
+  // zero or NaN case
+  if ((x == 0.0) || (x != x))
+    return x;
+
+  // negative case
+  bool neg = false;
+  if (x < 0.0) {
+    x = -x;
+    neg = true;
+  }
+
+  double y = std::floor(x + 0.5);
+  int i = static_cast<int>(y);
+  if ((y - x >= 0.5) && (i & 1))
+    --y;
+
+  return neg ? -y : y;
+}
+#endif // HAVE_RINT
+
+
 namespace itpp {
 
   // ----------------------------------------------------------------------
