@@ -8,7 +8,7 @@
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ namespace itpp {
   //! Style of assert, error and warning messages.
   enum error_msg_style { Full, Minimum };
 
-  //! Set preffered style of assert, error and warning messages
+  //! Set preferred style of assert, error and warning messages
   void it_error_msg_style(error_msg_style style);
 
 
@@ -160,7 +160,7 @@ namespace itpp {
   //! Print information message if NDEBUG is not defined
 #  define it_info_debug(s) ((void) 0)
   /*!
-    \brief Print information message withot \c std::endl at the end if
+    \brief Print information message without \c std::endl at the end if
     NDEBUG is not defined
   */
 #  define it_info_no_endl_debug(s) ((void) 0)
