@@ -9,7 +9,7 @@
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -435,7 +435,7 @@ namespace itpp {
 
     for (int i=0;i<a.rows();i++) {
       for (int j=0;j<a.cols();j++) {
-	temp(i,j)+=(std::complex<double>)a(i,j);
+        temp(i,j) += std::complex<double>(static_cast<double>(a(i,j)), 0.0);
       }
     }
     return temp;
@@ -461,7 +461,7 @@ namespace itpp {
 
     for (int i=0;i<a.rows();i++) {
       for (int j=0;j<a.cols();j++) {
-	temp(i,j)+=(std::complex<double>)a(i,j);
+        temp(i,j) += std::complex<double>(static_cast<double>(a(i,j)), 0.0);
       }
     }
     return temp;
@@ -474,7 +474,7 @@ namespace itpp {
 
     for (int i=0;i<a.rows();i++) {
       for (int j=0;j<a.cols();j++) {
-	temp(i,j)+=(std::complex<double>)a(i,j);
+        temp(i,j) += std::complex<double>(static_cast<double>(a(i,j)), 0.0);
       }
     }
     return temp;
