@@ -2,16 +2,13 @@
  * \file
  * \brief Sparse Matrix Class Definitions
  * \author Tony Ottosson and Tobias Ringstrom
- * 
- * $Date$
- * $Revision$
  *
  * -------------------------------------------------------------------------
  *
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +30,12 @@
 #ifndef SMAT_H
 #define SMAT_H
 
+#ifndef _MSC_VER
+#  include <itpp/config.h>
+#else
+#  include <itpp/config_msvc.h>
+#endif
+
 #include <itpp/base/svec.h>
 
 
@@ -49,7 +52,7 @@ namespace itpp {
 
   // ------------------------ Sparse_Mat Friends -------------------------------------
 
-  //! m1+m2 where m1 and m2 are sparse matrices  
+  //! m1+m2 where m1 and m2 are sparse matrices
   template <class T>
     Sparse_Mat<T> operator+(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);
 
@@ -93,12 +96,18 @@ namespace itpp {
   template <class T>
     Sparse_Mat<T> mult_trans(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);
 
-  /*! 
+  /*!
     \brief Templated Sparse Matrix Class
     \author Tony Ottosson and Tobias Ringstrom
 
-    A sparse matrix is a matrix where most elements are zero. The maximum number of
-    non-zero elements in each column is a parameter to the constructor.
+    A sparse matrix is a matrix where most elements are zero. The
+    maximum number of non-zero elements in each column is a parameter
+    to the constructor.
+
+    The implementation is based on representing all columns as sparse
+    vectors. Thus, column access generally is much faster than row
+    access. The elements in each vector are stored in random order,
+    i.e. they are not sorted.
   */
   template <class T>
     class Sparse_Mat {
@@ -107,7 +116,7 @@ namespace itpp {
     //! Default constructor
     Sparse_Mat();
 
-    /*! 
+    /*!
       \brief Initiate an empty sparse matrix
 
       A Sparse_Mat consists of colums that have the type Sparse_Vec. The maximum number of non-zero elements is each column
@@ -125,17 +134,17 @@ namespace itpp {
     //! Initiate a new sparse matrix from a dense matrix. The elements of \c m are copied into the new sparse matrix
     Sparse_Mat(const Mat<T> &m);
 
-    /*! 
+    /*!
       \brief Initiate a new sparse matrix from a dense matrix. Elements of \c m larger than \c epsilon are copied into the new sparse matrix.
-    
+
       \note If the type T is double complex, then the elements of \c m larger than \c abs(epsilon) are copied into the new sparse matrix.
     */
     Sparse_Mat(const Mat<T> &m, T epsilon);
 
     //! Destructor
     ~Sparse_Mat();
-  
-    /*! 
+
+    /*!
       \brief Set the size of the sparse matrix
 
       A Sparse_Mat consists of colums that have the type Sparse_Vec. The maximum number of non-zero elements is each column
@@ -146,7 +155,7 @@ namespace itpp {
       \param row_data_init The maximum number of non-zero elements in each column (default value -1 \c => allocated size for the data is not changed)
     */
     void set_size(int rows, int cols, int row_data_init=-1);
-  
+
     //! Returns the number of rows of the sparse matrix
     int rows() const { return n_rows; }
 
@@ -158,16 +167,16 @@ namespace itpp {
 
     //! Returns the density of the sparse matrix: (number of non-zero elements)/(total number of elements)
     double density();
-  
+
     //! Set the maximum number of non-zero elements in each column equal to the actual number of non-zero elements in each column
     void compact();
-  
+
     //! Returns a full, dense matrix in \c m
     void full(Mat<T> &m) const;
 
-    //! Returns a full, dense matrix 
+    //! Returns a full, dense matrix
     Mat<T> full() const;
-  
+
     //! Returns element of row \c r and column \c c
     T operator()(int r, int c) const;
 
@@ -207,51 +216,60 @@ namespace itpp {
     //! Returns column \c c of the Sparse_Mat in the Sparse_Vec \c v
     void get_col(int c, Sparse_Vec<T> &v) const;
 
-    //! Returns column \c c of the Sparse_Mat 
+    //! Returns column \c c of the Sparse_Mat
     Sparse_Vec<T> get_col(int c) const;
 
-    //! Set column \c c of the Sparse_Mat 
+    //! Set column \c c of the Sparse_Mat
     void set_col(int c, const Sparse_Vec<T> &v);
-  
-    //! Transpose the sparse matrix, return the result in \c m
+
+    /*! Transpose the sparse matrix, return the result in \c m
+
+    Note: this function can be slow for large matrices.
+     */
     void transpose(Sparse_Mat<T> &m) const;
 
-    //! Returns the transpose of the sparse matrix
+    /*! Returns the transpose of the sparse matrix
+
+    Note: this function can be slow for large matrices.
+    */
     Sparse_Mat<T> transpose() const;
 
-    //! Returns the transpose of the sparse matrix
+    /*! Returns the transpose of the sparse matrix
+
+    Note: this function can be slow for large matrices.
+    */
     // Sparse_Mat<T> T() const { return this->transpose(); };
-  
+
     //! Assign sparse matrix the value and dimensions of the sparse matrix \c m
     void operator=(const Sparse_Mat<T> &m);
 
     //! Assign sparse matrix the value and dimensions of the dense matrix \c m
     void operator=(const Mat<T> &m);
-  
+
     //! Returns the sign inverse of all elements in the sparse matrix
     Sparse_Mat<T> operator-() const;
-  
+
     //! Compare two sparse matricies. False if wrong sizes or different values
     bool operator==(const Sparse_Mat<T> &m) const;
-  
+
     //! Add sparse matrix \c v to all non-zero elements of the sparse matrix
     void operator+=(const Sparse_Mat<T> &v);
-  
+
     //! Add matrix \c v to all non-zero elements of the sparse matrix
     void operator+=(const Mat<T> &v);
-  
+
     //! Subtract sparse matrix \c v from all non-zero elements of the sparse matrix
     void operator-=(const Sparse_Mat<T> &v);
-  
+
     //! Subtract matrix \c v from all non-zero elements of the sparse matrix
     void operator-=(const Mat<T> &v);
-  
+
     //! Multiply all non-zero elements of the sparse matrix with the scalar \c v
     void operator*=(const T &v);
-  
+
     //! Divide all non-zero elements of the sparse matrix with the scalar \c v
     void operator/=(const T &v);
-  
+
     //! Addition m1+m2 where m1 and m2 are sparse matrices
     friend Sparse_Mat<T> operator+<>(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2);
 
@@ -290,24 +308,24 @@ namespace itpp {
     void alloc_empty();
     void alloc(int row_data_size=200);
     void free();
-  
+
     int n_rows, n_cols;
     Sparse_Vec<T> *col;
   };
 
-  /*! 
+  /*!
     \relates Sparse_Mat
     \brief Sparse integer matrix
   */
   typedef Sparse_Mat<int> sparse_imat;
 
-  /*! 
+  /*!
     \relates Sparse_Mat
     \brief Sparse double matrix
   */
   typedef Sparse_Mat<double> sparse_mat;
 
-  /*! 
+  /*!
     \relates Sparse_Mat
     \brief Sparse complex<double> matrix
   */
@@ -372,7 +390,7 @@ namespace itpp {
       n_rows = m.n_rows;
       n_cols = m.n_cols;
       alloc_empty();
-    
+
       for (int c=0; c<n_cols; c++)
 	col[c] = m.col[c];
     }
@@ -477,28 +495,28 @@ namespace itpp {
   template <class T>
     T Sparse_Mat<T>::operator()(int r, int c) const
     {
-      it_assert0(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
+      it_assert_debug(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
       return col[c](r);
     }
 
   template <class T>
     void Sparse_Mat<T>::set(int r, int c, T v)
     {
-      it_assert0(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
+      it_assert_debug(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
       col[c].set(r, v);
     }
 
   template <class T>
     void Sparse_Mat<T>::set_new(int r, int c, T v)
     {
-      it_assert0(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
+      it_assert_debug(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
       col[c].set_new(r, v);
     }
 
   template <class T>
     void Sparse_Mat<T>::add_elem(int r, int c, T v)
     {
-      it_assert0(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
+      it_assert_debug(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
       col[c].add_elem(r, v);
     }
 
@@ -506,13 +524,13 @@ namespace itpp {
     void Sparse_Mat<T>::zeros()
     {
       for (int c=0; c<n_cols; c++)
-	col[c].zeros();      
+	col[c].zeros();
     }
 
   template <class T>
     void Sparse_Mat<T>::zero_elem(const int r, const int c)
     {
-      it_assert0(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
+      it_assert_debug(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
       col[c].zero_elem(r);
     }
 
@@ -520,29 +538,29 @@ namespace itpp {
     void Sparse_Mat<T>::clear()
     {
       for (int c=0; c<n_cols; c++)
-	col[c].clear();      
+	col[c].clear();
     }
 
   template <class T>
     void Sparse_Mat<T>::clear_elem(const int r, const int c)
     {
-      it_assert0(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
+      it_assert_debug(r>=0&&r<n_rows&&c>=0&&c<n_cols, "Incorrect input indexes given");
       col[c].clear_elem(r);
     }
 
   template <class T>
-    void Sparse_Mat<T>::set_submatrix(int r1, int r2, int c1, int c2, const Mat<T>& m) 
+    void Sparse_Mat<T>::set_submatrix(int r1, int r2, int c1, int c2, const Mat<T>& m)
   {
     if (r1 == -1) r1 = n_rows-1;
     if (r2 == -1) r2 = n_rows-1;
     if (c1 == -1) c1 = n_cols-1;
     if (c2 == -1) c2 = n_cols-1;
 
-    it_assert1(r1>=0 && r2>=0 && r1<n_rows && r2<n_rows &&
+    it_assert_debug(r1>=0 && r2>=0 && r1<n_rows && r2<n_rows &&
 	       c1>=0 && c2>=0 && c1<n_cols && c2<n_cols, "Sparse_Mat<Num_T>::set_submatrix(): index out of range");
 
-    it_assert1(r2>=r1 && c2>=c1, "Sparse_Mat<Num_T>::set_submatrix: r2<r1 or c2<c1");
-    it_assert1(m.rows() == r2-r1+1 && m.cols() == c2-c1+1, "Mat<Num_T>::set_submatrix(): sizes don't match");
+    it_assert_debug(r2>=r1 && c2>=c1, "Sparse_Mat<Num_T>::set_submatrix: r2<r1 or c2<c1");
+    it_assert_debug(m.rows() == r2-r1+1 && m.cols() == c2-c1+1, "Mat<Num_T>::set_submatrix(): sizes don't match");
 
     for (int i=0 ; i<m.rows() ; i++) {
       for (int j=0 ; j<m.cols() ; j++) {
@@ -552,9 +570,9 @@ namespace itpp {
   }
 
   template <class T>
-    void Sparse_Mat<T>::set_submatrix(int r, int c, const Mat<T>& m) 
+    void Sparse_Mat<T>::set_submatrix(int r, int c, const Mat<T>& m)
   {
-    it_assert1(r>=0 && r+m.rows()<=n_rows &&
+    it_assert_debug(r>=0 && r+m.rows()<=n_rows &&
                c>=0 && c+m.cols()<=n_cols, "Sparse_Mat<Num_T>::set_submatrix(): index out of range");
 
     for (int i=0 ; i<m.rows() ; i++) {
@@ -567,9 +585,9 @@ namespace itpp {
   template <class T>
     Sparse_Mat<T> Sparse_Mat<T>::get_submatrix(int r1, int r2, int c1, int c2) const
     {
-      it_assert0(r1<=r2 && r1>=0 && r1<n_rows && c1<=c2 && c1>=0 && c1<n_cols,
+      it_assert_debug(r1<=r2 && r1>=0 && r1<n_rows && c1<=c2 && c1>=0 && c1<n_cols,
 		"Sparse_Mat<T>::get_submatrix(): illegal input variables");
-    
+
       Sparse_Mat<T> r(r2-r1+1, c2-c1+1);
 
       for (int c=c1; c<=c2; c++)
@@ -582,7 +600,7 @@ namespace itpp {
   template <class T>
     Sparse_Mat<T> Sparse_Mat<T>::get_submatrix_cols(int c1, int c2) const
     {
-      it_assert0(c1<=c2 && c1>=0 && c1<n_cols, "Sparse_Mat<T>::get_submatrix_cols()");
+      it_assert_debug(c1<=c2 && c1>=0 && c1<n_cols, "Sparse_Mat<T>::get_submatrix_cols()");
       Sparse_Mat<T> r(n_rows, c2-c1+1, 0);
 
       for (int c=c1; c<=c2; c++)
@@ -638,7 +656,7 @@ namespace itpp {
       n_rows = m.n_rows;
       n_cols = m.n_cols;
       alloc_empty();
-    
+
       for (int c=0; c<n_cols; c++)
 	col[c] = m.col[c];
     }
@@ -690,7 +708,7 @@ namespace itpp {
   template <class T>
     void Sparse_Mat<T>::operator+=(const Sparse_Mat<T> &m)
     {
-      it_assert0(m.rows()==n_rows&&m.cols()==n_cols, "Addition of unequal sized matrices is not allowed");
+      it_assert_debug(m.rows()==n_rows&&m.cols()==n_cols, "Addition of unequal sized matrices is not allowed");
 
       Sparse_Vec<T> v;
       for (int c=0; c<n_cols; c++) {
@@ -702,7 +720,7 @@ namespace itpp {
   template <class T>
     void Sparse_Mat<T>::operator+=(const Mat<T> &m)
     {
-      it_assert0(m.rows()==n_rows&&m.cols()==n_cols, "Addition of unequal sized matrices is not allowed");
+      it_assert_debug(m.rows()==n_rows&&m.cols()==n_cols, "Addition of unequal sized matrices is not allowed");
 
       for (int c=0; c<n_cols; c++)
 	col[c]+=(m.get_col(c));
@@ -711,7 +729,7 @@ namespace itpp {
   template <class T>
     void Sparse_Mat<T>::operator-=(const Sparse_Mat<T> &m)
     {
-      it_assert0(m.rows()==n_rows&&m.cols()==n_cols, "Subtraction of unequal sized matrices is not allowed");
+      it_assert_debug(m.rows()==n_rows&&m.cols()==n_cols, "Subtraction of unequal sized matrices is not allowed");
 
       Sparse_Vec<T> v;
       for (int c=0; c<n_cols; c++) {
@@ -723,7 +741,7 @@ namespace itpp {
   template <class T>
     void Sparse_Mat<T>::operator-=(const Mat<T> &m)
     {
-      it_assert0(m.rows()==n_rows&&m.cols()==n_cols, "Subtraction of unequal sized matrices is not allowed");
+      it_assert_debug(m.rows()==n_rows&&m.cols()==n_cols, "Subtraction of unequal sized matrices is not allowed");
 
       for (int c=0; c<n_cols; c++)
 	col[c]-=(m.get_col(c));
@@ -746,8 +764,8 @@ namespace itpp {
   template <class T>
     Sparse_Mat<T> operator+(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2)
     {
-      it_assert0(m1.n_cols == m2.n_cols && m1.n_rows == m2.n_rows , "Sparse_Mat<T> + Sparse_Mat<T>");
-    
+      it_assert_debug(m1.n_cols == m2.n_cols && m1.n_rows == m2.n_rows , "Sparse_Mat<T> + Sparse_Mat<T>");
+
       Sparse_Mat<T> m(m1.n_rows, m1.n_cols, 0);
 
       for (int c=0; c<m.n_cols; c++)
@@ -772,24 +790,36 @@ namespace itpp {
       return ret;
     }
 
-  // modified implementation by EGL, May'05
   template <class T>
     Sparse_Mat<T> operator*(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2)
     {
-      it_assert0(m1.n_cols == m2.n_rows, "Sparse_Mat<T> * Sparse_Mat<T>");
-      
+      it_assert_debug(m1.n_cols == m2.n_rows, "Sparse_Mat<T> * Sparse_Mat<T>");
+
       Sparse_Mat<T> ret(m1.n_rows, m2.n_cols);
-      
+
       for (int c=0; c<m2.n_cols; c++) {
-	for (int p2=0; p2<m2.col[c].nnz(); p2++) {
-	  Sparse_Vec<T> &mcol = m1.col[m2.col[c].get_nz_index(p2)];
+	Sparse_Vec<T> &m2colc=m2.col[c];
+	for (int p2=0; p2<m2colc.nnz(); p2++) {
+	  Sparse_Vec<T> &mcol = m1.col[m2colc.get_nz_index(p2)];
+	  T x = m2colc.get_nz_data(p2);
 	  for (int p1=0; p1<mcol.nnz(); p1++) {
 	    int r = mcol.get_nz_index(p1);
-	    T inc = mcol.get_nz_data(p1) * m2.col[c].get_nz_data(p2);
+	    T inc = mcol.get_nz_data(p1) *x;
 	    ret.col[c].add_elem(r,inc);
 	  }
 	}
       }
+      // old code
+/*       for (int c=0; c<m2.n_cols; c++) { */
+/* 	for (int p2=0; p2<m2.col[c].nnz(); p2++) { */
+/* 	  Sparse_Vec<T> &mcol = m1.col[m2.col[c].get_nz_index(p2)]; */
+/* 	  for (int p1=0; p1<mcol.nnz(); p1++) { */
+/* 	    int r = mcol.get_nz_index(p1); */
+/* 	    T inc = mcol.get_nz_data(p1) * m2.col[c].get_nz_data(p2); */
+/* 	    ret.col[c].add_elem(r,inc); */
+/* 	  } */
+/* 	} */
+/*       } */
       ret.compact();
       return ret;
     }
@@ -799,8 +829,8 @@ namespace itpp {
 /*   template <class T> */
 /*     Sparse_Mat<T> operator*(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2) */
 /*     { */
-/*       it_assert0(m1.n_cols == m2.n_rows, "Sparse_Mat<T> * Sparse_Mat<T>"); */
-    
+/*       it_assert_debug(m1.n_cols == m2.n_rows, "Sparse_Mat<T> * Sparse_Mat<T>"); */
+
 /*       Sparse_Mat<T> ret(m1.n_rows, m2.n_cols); */
 /*       ivec occupied_by(ret.n_rows), pos(ret.n_rows); */
 /*       for (int rp=0; rp<m1.n_rows; rp++) */
@@ -834,21 +864,21 @@ namespace itpp {
   template <class T>
     Sparse_Vec<T> operator*(const Sparse_Mat<T> &m, const Sparse_Vec<T> &v)
     {
-      it_assert0(m.n_cols == v.size(), "Sparse_Mat<T> * Sparse_Vec<T>");
+      it_assert_debug(m.n_cols == v.size(), "Sparse_Mat<T> * Sparse_Vec<T>");
 
       Sparse_Vec<T> ret(m.n_rows);
 
       /* The two lines below added because the input parameter "v" is
 	 declared const, but the some functions (e.g., nnz()) change
 	 the vector... Is there a better workaround? */
-      Sparse_Vec<T> vv;
-      vv = v;
+      Sparse_Vec<T> vv = v;
 
       for (int p2=0; p2<vv.nnz(); p2++) {
 	Sparse_Vec<T> &mcol = m.col[vv.get_nz_index(p2)];
+	T x = vv.get_nz_data(p2);
 	for (int p1=0; p1<mcol.nnz(); p1++) {
 	  int r = mcol.get_nz_index(p1);
-	  T inc = mcol.get_nz_data(p1) * vv.get_nz_data(p2);
+	  T inc = mcol.get_nz_data(p1) * x;
 	  ret.add_elem(r,inc);
 	}
       }
@@ -860,8 +890,8 @@ namespace itpp {
   template <class T>
     Vec<T> operator*(const Sparse_Mat<T> &m, const Vec<T> &v)
     {
-      it_assert0(m.n_cols == v.size(), "Sparse_Mat<T> * Vec<T>");
-    
+      it_assert_debug(m.n_cols == v.size(), "Sparse_Mat<T> * Vec<T>");
+
       Vec<T> r(m.n_rows);
       r.clear();
 
@@ -869,15 +899,15 @@ namespace itpp {
 	for (int p=0; p<m.col[c].nnz(); p++)
 	  r(m.col[c].get_nz_index(p)) += m.col[c].get_nz_data(p) * v(c);
       }
-      
+
       return r;
     }
 
   template <class T>
     Vec<T> operator*(const Vec<T> &v, const Sparse_Mat<T> &m)
     {
-      it_assert0(v.size() == m.n_rows, "Vec<T> * Sparse_Mat<T>");
-    
+      it_assert_debug(v.size() == m.n_rows, "Vec<T> * Sparse_Mat<T>");
+
       Vec<T> r(m.n_cols);
       r.clear();
 
@@ -931,8 +961,8 @@ namespace itpp {
   template <class T>
     Sparse_Mat<T> trans_mult(const Sparse_Mat<T> &m1, const Sparse_Mat<T> &m2)
     {
-      it_assert0(m1.n_rows == m2.n_rows, "trans_mult()");
-    
+      it_assert_debug(m1.n_rows == m2.n_rows, "trans_mult()");
+
       Sparse_Mat<T> ret(m1.n_cols, m2.n_cols);
       Vec<T> col;
       for (int c=0; c<ret.n_cols; c++) {
@@ -960,6 +990,7 @@ namespace itpp {
       return trans_mult(m1.transpose(),m2.transpose());
     }
 
+  //! Convert a dense matrix \c m into its sparse representation
   template <class T>
     inline Sparse_Mat<T> sparse(const Mat<T> &m, T epsilon)
     {
@@ -967,6 +998,7 @@ namespace itpp {
       return s;
     }
 
+  //! Convert a sparse matrix \c s into its dense representation
   template <class T>
     inline Mat<T> full(const Sparse_Mat<T> &s)
     {
@@ -975,6 +1007,7 @@ namespace itpp {
       return m;
     }
 
+  //! Transpose a sparse matrix \c s
   template <class T>
     inline Sparse_Mat<T> transpose(const Sparse_Mat<T> &s)
     {
@@ -983,84 +1016,57 @@ namespace itpp {
       return m;
     }
 
-  /*--------------------------------------------------------------
-   * Explicit initializations
-   *--------------------------------------------------------------*/
+  //! \cond
 
-#ifndef _MSC_VER
+  // ---------------------------------------------------------------------
+  // Instantiations
+  // ---------------------------------------------------------------------
+
+#ifdef HAVE_EXTERN_TEMPLATE
 
-  //! Template instantiation of Sparse_Mat<int>
   extern template class Sparse_Mat<int>;
-  //! Template instantiation of Sparse_Mat<double>
   extern template class Sparse_Mat<double>;
-  //! Template instantiation of Sparse_Mat<complex<double> >
   extern template class Sparse_Mat<std::complex<double> >;
 
-
-  //! Template instantiation of operator+
   extern template sparse_imat operator+(const sparse_imat &, const sparse_imat &);
-  //! Template instantiation of operator+
   extern template sparse_mat operator+(const sparse_mat &, const sparse_mat &);
-  //! Template instantiation of operator+
   extern template sparse_cmat operator+(const sparse_cmat &, const sparse_cmat &);
 
-  //! Template instantiation of operator*
   extern template sparse_imat operator*(const sparse_imat &, const sparse_imat &);
-  //! Template instantiation of operator*
   extern template sparse_mat operator*(const sparse_mat &, const sparse_mat &);
-  //! Template instantiation of operator*
   extern template sparse_cmat operator*(const sparse_cmat &, const sparse_cmat &);
 
-  //! Template instantiation of operator*
   extern template ivec operator*(const ivec &, const sparse_imat &);
-  //! Template instantiation of operator*
   extern template vec operator*(const vec &, const sparse_mat &);
-  //! Template instantiation of operator*
   extern template cvec operator*(const cvec &, const sparse_cmat &);
 
-  //! Template instantiation of operator*
   extern template ivec operator*(const sparse_imat &, const ivec &);
-  //! Template instantiation of operator*
   extern template vec operator*(const sparse_mat &, const vec &);
-  //! Template instantiation of operator*
   extern template cvec operator*(const sparse_cmat &, const cvec &);
 
-  //! Template instantiation of trans_mult
   extern template imat trans_mult(const sparse_imat &);
-  //! Template instantiation of trans_mult
   extern template mat trans_mult(const sparse_mat &);
-  //! Template instantiation of trans_mult
   extern template cmat trans_mult(const sparse_cmat &);
 
-  //! Template instantiation of trans_mult_s
   extern template sparse_imat trans_mult_s(const sparse_imat &);
-  //! Template instantiation of trans_mult_s
   extern template sparse_mat trans_mult_s(const sparse_mat &);
-  //! Template instantiation of trans_mult_s
   extern template sparse_cmat trans_mult_s(const sparse_cmat &);
 
-  //! Template instantiation of trans_mult
   extern template sparse_imat trans_mult(const sparse_imat &, const sparse_imat &);
-  //! Template instantiation of trans_mult
   extern template sparse_mat trans_mult(const sparse_mat &, const sparse_mat &);
-  //! Template instantiation of trans_mult
   extern template sparse_cmat trans_mult(const sparse_cmat &, const sparse_cmat &);
 
-  //! Template instantiation of trans_mult
   extern template ivec trans_mult(const sparse_imat &, const ivec &);
-  //! Template instantiation of trans_mult
   extern template vec trans_mult(const sparse_mat &, const vec &);
-  //! Template instantiation of trans_mult
   extern template cvec trans_mult(const sparse_cmat &, const cvec &);
 
-  //! Template instantiation of mult_trans
   extern template sparse_imat mult_trans(const sparse_imat &, const sparse_imat &);
-  //! Template instantiation of mult_trans
   extern template sparse_mat mult_trans(const sparse_mat &, const sparse_mat &);
-  //! Template instantiation of mult_trans
   extern template sparse_cmat mult_trans(const sparse_cmat &, const sparse_cmat &);
 
-#endif
+#endif // HAVE_EXTERN_TEMPLATE
+
+  //! \endcond
 
 } // namespace itpp
 
