@@ -8,7 +8,7 @@
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -301,6 +301,8 @@ namespace itpp {
     //! Initialize the generator
     virtual void init() { init_flag = true; }
 
+    using Fading_Generator::generate;
+
     //! Generate \a no_samples values from the fading process
     virtual void generate(int no_samples, cvec& output);
   };
@@ -325,6 +327,8 @@ namespace itpp {
     //! Initialize the generator
     virtual void init();
 
+    using Fading_Generator::generate;
+
     //! Generate \a no_samples values from the fading process
     virtual void generate(int no_samples, cvec& output);
 
@@ -373,6 +377,8 @@ namespace itpp {
     //! Initialize the generator
     virtual void init() = 0;
 
+    using Fading_Generator::generate;
+
     //! Generate \a no_samples values from the fading process
     virtual void generate(int no_samples, cvec& output) = 0;
 
@@ -445,6 +451,8 @@ namespace itpp {
     //! Initialize the generator
     virtual void init();
 
+    using Correlated_Fading_Generator::generate;
+
     //! Generate \a no_samples values from the fading process
     virtual void generate(int no_samples, cvec &output);
 
@@ -501,6 +509,8 @@ namespace itpp {
     //! Initialize the generator
     virtual void init();
 
+    using Correlated_Fading_Generator::generate;
+
     //! Generate \a no_samples values from the fading process
     virtual void generate(int no_samples, cvec &output);
 
@@ -560,6 +570,8 @@ namespace itpp {
     //! Initialize the generator
     virtual void init() { init_flag = true; }
 
+    using Correlated_Fading_Generator::generate;
+
     //! Generate \a no_samples values from the fading process
     virtual void generate(int no_samples, cvec &output);
 
