@@ -8,7 +8,7 @@
  * IT++ - C++ library of mathematical, signal processing, speech processing,
  *        and communications classes and functions
  *
- * Copyright (C) 1995-2007  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,9 +85,25 @@ namespace itpp {
 
     //! Returns number of bits per symbol
     virtual int bits_per_symbol() const { return k; }
-    //! Get the symbol values used in the modulator.
+    //! Get the symbol values used in the modulator
     virtual Vec<T> get_symbols() const { return symbols; }
-    //! Get the bitmap in the decimal form
+    /*!
+     * \brief Get the bitmap, which maps input bits into symbols
+     *
+     * The mapping is done as follows. An input bit sequence in decimal
+     * notation is used for indexing the \c bits2symbols table. The indexing
+     * result denotes the symbol to be used from the \c symbols table, e.g.:
+     *
+     * \code
+     * PSK mod(8); // assume 8-PSK modulator
+     * cvec sym =  mod.get_symbols();
+     * ivec bits2sym = mod.get_bits2symbols();
+     * bvec in_bits = "100" // input bits
+     * int d = bin2dec(in_bits); // decimal representation of in_bits = 4
+     * // mapping of d into PSK symbol using bits2sym and sym tables
+     * std::complex<double> out_symbol = sym(bits2sym(d));
+     * \endcode
+     */
     virtual ivec get_bits2symbols() const { return bits2symbols; }
 
     //! Modulation of symbols
@@ -304,7 +320,7 @@ namespace itpp {
     k = levels2bits(M);
     bitmap.set_size(M, k);
     for (int m = 0; m < M; m++) {
-      bitmap.set_row(m, dec2bin(k, bits2symbols(m)));
+      bitmap.set_row(bits2symbols(m), dec2bin(k, m));
     }
     calculate_softbit_matrices(bits2symbols);
     setup_done = true;
